import Base from "comptechsoft-vue-foundation/src/Base";

class Axios extends Base {
    
    constructor(name, parent = null, onInit = null) {
        super(name, parent, onInit)

        if(document.head.querySelector('meta[name="csrf-token"]') == null)
        {
            throw new Error('no csrf-token found.')
        }

        if(document.head.querySelector('meta[name="base-url"]') == null)
        {
            throw new Error('no base-url found.')
        }

        this.AddProperty('token', document.head.querySelector('meta[name="csrf-token"]').content) 
        this.AddProperty('url', document.head.querySelector('meta[name="base-url"]').content)
        this.AddProperty('endpoint')                // La ce endpoint se face requestul
        this.AddProperty('data')                    // Cu ce date se face requestul
        this.AddProperty('processing', false)       // Indicator de procesare: true => se proceseaza, false => nu se proceseaza
        this.AddProperty('success', false)          // Daca sa terminat sau nu cu succes: null | false | true
        this.AddProperty('error')                   
        this.AddProperty('response')                  
        this.AddProperty('interval', 250) 
        this.AddProperty('timeout', null) 

        this.AddProperty('onSuccess', data => {
            // console.log('Axisos::onSuccess()', data)
        }) 
        this.Set('onError', error => {
            // console.log(error)
        }) 

        axios.defaults.headers.common['X-Requested-With'] = 'XMLHttpRequest'
        axios.defaults.headers.common['X-CSRF-TOKEN'] = this.token
    }

    Endpoint(endpoint = null) {
        if( endpoint )
        {
            this.Set('endpoint', ((endpoint.substr(0,4) === 'http') ? '' : this.url + '/') + endpoint)
            return this
        }
        return this.endpoint
    }

    SetData(data) {
        this.Set('data', data)
        return this
    }

    Success(onSuccess) {
        this.Set('onSuccess', onSuccess)
        return this
    }

    onsuccess() {
        if(_.isFunction(this.onSuccess) )
        {
            this.onSuccess(this.response)
        }
    }

    onerror() {
        if(_.isFunction(this.onError) )
        {
            this.onError(this.error)
        }
    }
 
    makepost() {
        this.Set('processing', true)    // se marcheaza ca incepand efectuarea requestului
        
        this.Set('error', null)         // se reseteaza ultima eroare
        this.Set('success', null)       // se reseteaza indicatorul de succes

        axios.request({
            method: 'post', 
            url: this.Endpoint(),
            data: this.data,
        })
        .then( response => {
            this.Set('processing', false) 
            this.Set('success', true) 
            this.Set('response', response.data) 
            this.onsuccess()
        })
        .catch( error => {
            this.Set('processing', false) 
            this.Set('success', false) 
            this.Set('error', error)
            this.onerror()
        })
    }

    Post() {
        if( ! this.Endpoint() )
        {
            throw new Error('No endpoint.')
        }
        if( this.timeout )
        {
            clearTimeout(this.timeout)
        }
        this.timeout = setTimeout( () => {
            this.makepost()
        }, this.interval)
    }

    Processing() {
        return this.processing
    }
}

export default Axios