import Axios from 'comptechsoft-vue-foundation/src/Classes/Axios/Axios'
import Collection from 'comptechsoft-vue-foundation/src/Classes/Collection/Collection'
import Searchable from 'comptechsoft-vue-foundation/src/Classes/Fetcher/Searchable'
import Conditions from 'comptechsoft-vue-foundation/src/Classes/Fetcher/Conditions'

class Fetcher extends Axios {

    constructor(name, parent = null, onInit = null) {
        super(name, parent, onInit)

        this.Set('processing', true)
        this.AddProperty('populate_count', 0)
        this.AddProperty('current_page', 1)
        this.AddProperty('per_page')
        this.AddProperty('page_sizes', [1, 2, 3, 4, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 100, 200, 500])
        this.AddProperty('current_order')
        this.AddProperty('orderable', new Collection('orderable'))
        this.AddProperty('searchable', new Searchable('searchable'))
        this.AddProperty('conditions', new Conditions('conditions'))

        this.AddProperty('selected', [])
        this.AddProperty('detailed', [])
    }

    /**
     * Setters
     */
    SetPerPage(per_page) {
        this.per_page = per_page
        return this
    }

    SetSearchBy(fields) {
        this.searchable.SetSearchBy(fields) 
        return this
    }

    SetOrderable(key, type, fields) {
        this.orderable.Put(key, {
            fields,
            type,
            direction: null,
        })
        return this
    }

    SetCondition(field, condition) {
        this.conditions.SetCondition(field, condition)
        return this
    }

    /**
     * Getters
     */
    Endpoint(endpoint = null)
    {
        if(endpoint)
        {
            return super.Endpoint(endpoint)
        }
        if(! this.current_page)
        {
            return this.endpoint
        }
        return this.endpoint + '?page=' + this.current_page 
    } 
        
    RecordCount() {
        return ! this.response ? null : this.response.record_count
    }

    CurrentPage() {
        return ! this.response ? null : this.response.paginator.current_page
    }

    PerPage() {
        return ! this.response ? null : this.response.paginator.per_page
    }

    From() {
        return ! this.response ? null : this.response.paginator.from
    }

    To() {
        return ! this.response ? null : this.response.paginator.to
    }

    Total() {
        return ! this.response ? null : this.response.paginator.total
    }

    LastPage() {
        return ! this.response ? null : this.response.paginator.last_page
    }

    Records() {
        return ! this.response ? [] : this.response.paginator.data
    }

    HasOrderable(key) {
        return this.orderable.Has(key)
    }

    GetCurrentOrder() {
        let r = this.current_order ? this.orderable.Get(this.current_order) : null
        if( ! r )
        {
            return null
        }
        if( ! r.direction )
        {
            r.direction = 'asc'
        }
        return r
    }

    IsOrderedByField(key, direction) {
        if( this.current_order == null)
        {
            return false
        }
        if( this.current_order != key )
        {
            return false
        }
        return this.GetCurrentOrder().direction == direction
    }

    /**
     * Methods
     */
    ChangePerPage(per_page = null) {
        this.Set('current_page', 1)
        this.Post()
    }

    ChangePage(page) {
        this.Set('current_page', page)
        this.Post()
    }

    SearchBy(value = null) {
        if(value)
        {
            this.searchable.Set('value', value)
        }
        this.Set('current_page', 1)
        this.Post()
    }

    OrderBy(key, direction) {
        this.Set('current_order', key)
        this.orderable.items[key].direction = direction
        this.Post()
    }

    FilterBy() {
        this.Set('current_page', 1)
        this.Post()
    }

    /** Selection */
    SelectAll(flag) {
        _.each(this.Records(), record => {
            this.SelectRecord(flag, record)
        })
        return this
    }

    AllRecordsAreSelected() {
        let records = this.Records()
        if(records.length == 0)
        {
            return false
        }
        let r = true
        _.each(records, record => {
            if(! this.RecordIsSelected(record) )
            {
                r = false
            }
        })
        return r
    }

    RecordIsSelected(record) {
        let r = _.find(this.selected, item => item == record.id)
        return r != undefined
    }

    SelectRecord(flag, record) {
        if(flag)
        {
            if(! this.RecordIsSelected(record) )
            {
                this.selected.push(record.id)
            }
        }
        else
        {
            if(this.RecordIsSelected(record))
            {
                let r = [...this.selected]
                _.remove(r, item => record.id == item)
                this.Set('selected', r)
            }
        }
        return this
    }

    GetSelected() {
        return this.selected
    }

    /** Details */
    ToggleDetails(record) {
        if(! this.RecordIsDetailed(record))
        {
            this.detailed.push(record.id)
        }
        else
        {
            let r = [...this.detailed]
            _.remove(r, item => record.id == item)
            this.Set('detailed', r)
        }
        return this
    }

    RecordIsDetailed(record) {
        let r = _.find(this.detailed, item => item == record.id)
        return r != undefined
    }

    GetDetailed() {
        return this.detailed
    }

    /** Make the request */
    Post() {
        this.SetData({
            per_page: this.per_page,
            searchable: this.searchable.Get(),
            orderable: this.GetCurrentOrder(),
            conditions: this.conditions.GetConditions(),
        })
        super.Post()
    }

}

export default Fetcher