import Base from "comptechsoft-vue-foundation/src/Base";
import Layout from "./../Components/Layout/Layout";

class Component extends Base {

    constructor(name, parent = null, onInit = null) {

        super(name, parent, onInit)

        this.AddProperty('id', name)

        this.AddProperty('enabled', true)
        this.AddProperty('visible', true)

        /** Layout */
        this.AddProperty('layout', new Layout('layout', this) )

        /**  Events */
        this.AddProperty('onHide', null)
        this.AddProperty('onShow', null)
        this.AddProperty('onClick', null)
    }

    /**
     * Layout: caption, icon, classes and styles
     */

    /** Layout. Setters */

    SetCaption(caption) {
        this.layout.Set('caption', caption)
        return this
    }

    SetTitle(title) {
        this.layout.SetTitle(title)
        return this
    }

    SetIcon(icon) {
        this.layout.SetIcon(icon)
        return this
    }

    AddClass(key) {
        this.layout.AddClass(key)
        return this
    }

    RemoveClass(key) {
        this.layout.RemoveClass(key)
        return this
    }

    AddStyle(style, value) {
        this.layout.AddStyle(style, value)
        return this
    }

    /** Layout. Getters  */

    Icon() {
        return this.layout.GetPropertyValue('icon')
    }

    Caption() {
        return this.layout.GetPropertyValue('caption')
    }

    Title() {
        return this.layout.GetPropertyValue('title')
    }

    Disabled() {
        return this.Set('enabled', false)
    }

    Enabled() {
        return this.Set('enabled', true)
    }

    HtmlClass() {
        return this.layout.getHtmlClass()
    }

    HtmlStyle() {
        return this.layout.getHtmlStyle()
    }

    /**
     * Utils
     */
    // GetHighestStyleProperty(jqSelector, property) {
    //     let index_highest = 0
    //     $(jqSelector).each(function() {
    //         var index_current = parseInt($(this).css(property), 10);
    //         if(index_current > index_highest) 
    //         {
    //             index_highest = index_current
    //         }
    //     });
    //     return index_highest
    // }

    /**
     * Show | Hide
     */
    IsVisible() {
        return this.GetPropertyValue('visible') === true
    }

    /** Methods */
    Init() {
        if( _.isFunction(this.onInit) )
        {
            this.onInit()
        }
    }

    // OnHide() {
    //     console.log('on Hide (' + this.name + ')')
    // }

    Hide() {
        this.Set('visible', false)
        if( _.isFunction(this.onHide) )
        {
            this.onHide()
        }
        return this
    }

    // OnShow() {
    //     console.log('On Show (' + this.name + ')')
    // }

    Show() {
        this.Set('visible', true)
        if( _.isFunction(this.onShow) )
        {
            this.onShow()
        }
        return this
    }

    // Activate() {
    //     console.log('Activate (' + this.name + ')')
    // }

    Click(vue = null) {

        if( vue === null)
        {
            this.onError('The vue instance is not passed', this.name + '::Click()')
        }

        if( _.isFunction(this.onClick) )
        {
            this.onClick(vue)
        }
        vue.$emit('click', this)
    }

}

export default Component