import Component from "./../Component"
import Collection from "./../../../Classes/Collection/Collection"
import Column from './Column'
import Actions from "./Actions"

/**
 * +------------------+
 * | Datatable Object |
 * +------------------+
 * 
 *      Properties
 *      ==========
 * 
 *          columns:Collection                  - Coloanele
 *          actions:Actions                     - Actiunile din coloana "Actions"
 *          has_selectable_column:Boolean       - Daca are sau nu coloana checkbox "Select"
 *          has_details_column: Boolean         - Daca are sau nu coloana "Details"
 *          details_component:String            - Numele componentei ce se afiseaza la Details
 * 
 *      Methods
 *      =======
 *          AddColumn(column:Column)
 *          AddAction(action:Action)
 *          SetActionColumnCaption(caption:String)
 *          SetActionColumnIcon(icon:String)
 *          SetActionColumnWidth(width:Number)
 * 
 *          Headers()
 *          ColumnSources()
 *          Actions()
 *          Width()
 *          ColumnsCount()
 */

class Datatable extends Component {

    constructor(name, parent = null, onInit = null) {
        super(name, parent, onInit)
        
        this.AddProperty('columns', new Collection('columns', this))
        this.AddProperty('actions', new Actions('actions'))

        /**
         * Icoana default la coloana Actions
         */
        this.SetActionColumnIcon('la la-bars')

        /**
         * Latimea default la coloana Actions
         */
        this.SetActionColumnWidth(5) 

        this.AddProperty('has_selectable_column', true)
        this.AddProperty('has_details_column', true)
        this.AddProperty('details_component')
        this.AddProperty('messages', {
            no_records: null,
        })
    }


    AddColumn(column) {
        if(! _.isObject(column) || ! (column instanceof Column))
        {
            throw new Error('Column must be instance of Column Object.')
        }
        column.SetWidthStyle()
        this.columns.Put(column.name, column)
        return this
    }

    AddAction(action) {
        this.actions.AddAction(action)
        return this
    }

    SetActionColumnCaption(caption) {
        this.actions.SetCaption(caption)
        return this
    }

    SetActionColumnIcon(icon) {
        this.actions.SetIcon(icon)
        return this
    }

    SetActionColumnWidth(width) {
        this.actions.SetWidthStyle(width)
        return this
    }

    Headers() {
        let r = {}
        _.each(this.columns.All(), (column, key) => {
            if(column.IsVisible())
            {
                r[key] = column.header
            }
        })
        return r
    }

    ColumnSources() {
        let r = {}
        _.each(this.columns.All(), (column, key) => {
            if(column.IsVisible())
            {
                r[key] = column.source
            }
        })
        return r
    }

    Actions() {
        return this.actions.actions
    }

    Width() {
        let w = this.actions.width + (this.has_selectable_column ? 5 : 0) + + (this.has_details_column ? 5 : 0)
        _.each(this.columns.All(), column => {
            if(column.IsVisible()) 
            {
                w = w + column.width
            }
        })
        return w
    }

    ColumnsCount() {
        let r = 0
        _.each(this.columns.All(), (column, key) => {
            if(column.IsVisible())
            {
                r++
            }
        })
        if(this.has_selectable_column)
        {
            r++
        }
        if(this.has_details_column)
        {
            r++
        }
        if(this.Actions().Count())
        {
            r++
        }
        return r
    }

}

export default Datatable