import FormGroupTextbox from "comptechsoft-vue-foundation/src/Vue/Components/Forms/Controls/BsFormGroup/Textbox"

class FormGroupUploadUppy extends FormGroupTextbox {

    constructor(name, parent = null, onInit = null) {

        super(name, parent, onInit)

        this.AddProperty('instance', null)
        this.AddProperty('url', document.head.querySelector('meta[name="base-url"]').content)
        this.AddProperty('endpoint', null)

        this.AddProperty('autoProceed', true)
        this.AddProperty('allowMultipleUploads', false)
        this.AddProperty('showProgressDetails', true)
        this.AddProperty('maxFileSize', 3000000)
        this.AddProperty('maxNumberOfFiles', 1)
        this.AddProperty('minNumberOfFiles', 1)
    }

    CreateFileInput() {
        let jq = $('#' + this.textbox.id + ' .kt-uppy__wrapper')
        if(jq.length == 1)
        {
            this.instance.use(Uppy.FileInput, { 
                target: '#' + this.textbox.id + ' .kt-uppy__wrapper', 
                pretty: false 
            })
        }
    }

    CreateInformer() {
        let jq = $('#' + this.textbox.id + ' .kt-uppy__informer')
        if(jq.length == 1)
        {
            this.instance.use(Uppy.Informer, { 
                target: '#' + this.textbox.id + ' .kt-uppy__informer'  
            })
        }
    }

    CreateTus() {
        this.instance.use(Uppy.XHRUpload, { 
            headers: {
                'X-CSRF-TOKEN': document.head.querySelector('meta[name="csrf-token"]').content
            },
            endpoint: this.url + '/' + this.endpoint,
            limit: this.maxNumberOfFiles,
            formData: true,
            fieldName: 'files[]'
        })
    }

    CreateStatusBar() {
        let jq = $('#' + this.textbox.id + ' .kt-uppy__status')
        if(jq.length == 1)
        {
            this.instance.use(Uppy.StatusBar, {
                target: '#' + this.textbox.id + ' .kt-uppy__status',
                hideUploadButton: true,
                hideAfterFinish: false
            });
        }
    }

    FormatFileInput() {
        let jq =  $('#' + this.textbox.id + ' .uppy-FileInput-input')
        if(jq.length == 1)
        {
            jq.addClass('kt-uppy__input-control').attr('id', this.textbox.id + '_input_control')
        }
        $('#' + this.textbox.id + ' .uppy-FileInput-container')
            .append('<label class="kt-uppy__input-label btn btn-label-brand btn-bold btn-font-sm" for="' + (this.textbox.id + '_input_control') + '">Attach files</label>')
    }

    SetOnUpload() {
        let self = this
        this.instance.on('upload', function(data) {
            console.log('UPLOAD', data)

            $('#' + self.textbox.id + ' .kt-uppy__input-label').text("Uploading...");
            $('#' + self.textbox.id  + ' .kt-uppy__status').addClass('kt-uppy__status--ongoing');
            $('#' + self.textbox.id  + ' .kt-uppy__status').removeClass('kt-uppy__status--hidden');
        });
    }

    SetOnComplete() {
        let FileSize = require('file-size')
        let self = this
        this.instance.on('complete', function(file) {
            console.log('COMPLETE', file, file.successful)
            $.each(file.successful, function(index, value){
			
                let uploadListHtml = '<div class="kt-uppy__list-item" data-id="' + value.id + '"><div class="kt-uppy__list-label">' + value.name + ' (' + FileSize(value.size).human('si') + ')</div><span class="kt-uppy__list-remove" data-id="' + value.id + '"><i class="flaticon2-cancel-music"></i></span></div>';
                $('#' + self.textbox.id  + ' .kt-uppy__list').append(uploadListHtml);

                self.SetValue(value.response.body)
            });

            $('#' + self.textbox.id + ' .kt-uppy__input-label').text("Add more files");		

            $('#' + self.textbox.id  + ' .kt-uppy__status').addClass('kt-uppy__status--hidden');
            $('#' + self.textbox.id  + ' .kt-uppy__status').removeClass('kt-uppy__status--ongoing');
        });
    }

    CreateUppyCoreInstance() {
        this.instance = Uppy.Core({
            debug: true, 
            id: 'uppy-' + this.textbox.id,
            autoProceed: this.autoProceed,
            allowMultipleUploads: this.allowMultipleUploads,
            showProgressDetails: this.showProgressDetails,
            restrictions: {
                maxFileSize: this.maxFileSize, 
                maxNumberOfFiles: this.maxNumberOfFiles,
                minNumberOfFiles: this.minNumberOfFiles,
            }
        });
        this.CreateFileInput()
        this.CreateInformer()
        this.CreateTus()
        this.CreateStatusBar()

        this.FormatFileInput()

        this.SetOnUpload()
        this.SetOnComplete()

        this.instance.on('file-added', (file) => {
            console.log('Added file', file)
          })
    }

    InitUppy() {
        if(! this.instance)
        {
            let i = setInterval( () => {
                let jq = $('#' + this.textbox.id)
                if( jq.length == 1) 
                {
                    clearInterval(i)
                    this.CreateUppyCoreInstance()
                }
            }, 10)
        }
    }
}

export default FormGroupUploadUppy