import Portlet from 'comptechsoft-vue-foundation/src/Vue/Components/Metronic/Portlets/Portlet'
import Form from 'comptechsoft-vue-foundation/src/Vue/Components/Forms/Form'
import Collection from 'comptechsoft-vue-foundation/src/Classes/Collection/Collection'
import Action from 'comptechsoft-vue-foundation/src/Vue/Components/Metronic/Portlets/Crud/Action'

class CrudPortlet extends Portlet {

    constructor(name, parent = null, onInit = null) {

        super(name, parent, onInit)

        this.Hide()

        this.AddProperty('controls_component')

        this.AddProperty('current_action')
        this.AddProperty('blank', {})
        this.AddProperty('record', {})
        this.AddProperty('old', {})
        this.AddProperty('form', new Form('form'))
        this.AddProperty('actions', new Collection('actions'))

    }

    AddAction(action) {
        if(! _.isObject(Action) || ! (action instanceof Action))
        {
            throw new Error('Action must be instance of Action Object.')
        }
        this.actions.Put(action.name, action)
        return this
    }

    SetBlankRecord(blank) {
        return this.Set('blank', blank)
    }

    SetRecord(record) {
        let r = ((record != null) ? record : this.blank)
        _.each(r, (value, field) => {
            Vue.set(this.record, field, value)
            if( this.form.controls.hasOwnProperty(field) )
            {
                this.form.controls[field].SetValue(value)
            }
        })
        return this
    }

    ResetRecord() {
        _.each(this.blank, (value, field) => {
            Vue.set(this.record, field, value)
            if( this.form.controls.hasOwnProperty(field) )
            {
                this.form.controls[field].SetValue(value)
            }
        })
        return this
    }

    SetOld(record) {
        let r = ((record != null) ? record : this.blank)
        _.each(r, (value, field) => {
            Vue.set(this.old, field, value)
        })
        return this
    }

    DisableControls() {
        _.each(this.form.controls, (control, field) => {
            control.Disabled()
        })
        return this
    }

    EnableControls() {
        _.each(this.form.controls, (control, field) => {
            control.Enabled()
        })
        return this
    }

    ShowForm(action, record, onShowForm = null) {
        if(! this.IsVisible())
        {
            this.Set('current_action', action)
            this.SetRecord(record)
            this.SetOld(record)
            this.SetCaption(this.GetCaption())
            if(_.isFunction(onShowForm))
            {
                onShowForm(record, this)
            }
            if(action == 'delete')
            {
                this.DisableControls()
            }
            else
            {
                this.EnableControls()
            }
            this.Show()
        }
        return this
    }

    HideForm() {
        if( this.IsVisible())
        {
            this.form.ResetErrors()
            this.form.ResetResult()
            this.Set('current_action', null)
            this.ResetRecord()
            this.SetCaption(null)
            this.Hide()
        }
        return this
    }

    GetCurrentAction() {
        return this.current_action
    }

    GetCaption() {
        if( ! this.current_action )
        {
            return null
        }
        let action = this.actions.Get(this.current_action)
        if( _.isString(action.layout.caption) )
        {
            return action.layout.caption
        }
        if( _.isFunction(action.layout.caption) )
        {
            return action.layout.caption(this.old)
        }
    }

    GetButtonCaption() {
        if( ! this.current_action )
        {
            return null
        }
        let action = this.actions.Get(this.current_action)
        if( _.isString(action.button.layout.caption) )
        {
            return action.button.layout.caption
        }
        if( _.isFunction(action.button.layout.caption) )
        {
            return action.button.layout.caption.caption(this.old)
        }
    }

    GetButtonHtmlClass() {
        if( ! this.current_action )
        {
            return null
        }
        let action = this.actions.Get(this.current_action)
        return action.button.HtmlClass()
    }

    GetButtonIcon() {
        if( ! this.current_action )
        {
            return null
        }
        let action = this.actions.Get(this.current_action)
        return action.button.layout.icon
    }

    GetEndpoint() {
        if( ! this.current_action )
        {
            return null
        }
        let action = this.actions.Get(this.current_action)
        return action.endpoint
    }

    UpdateRecord(values) {
        _.each(values, (value, field) => {
            if( ! this.record.hasOwnProperty(field) )
            {
            }
            Vue.set(this.record, field, value)
        })
        return this
    }

    Submit() {
        let endpoint = this.GetEndpoint()
        this.form.SetEndpoint(endpoint).Submit(this.record)
    }
}

export default CrudPortlet