import Collection from "../Classes/Collection/Collection"

class BaseComponent {

    constructor(name) {

        this.name = name

        this.component = null
        
        /**
         * HTML id
         */
        this.id = name

        /**
         * HTML class
         */
        this.htmlClass = new Collection()

        /**
         * HTML style
         */
        this.htmlStyle = new Collection()

        /**
         * Visibility
         */
        this.visible = true

        /**
         * Enabled | Disabled
         */
        this.enabled = true

        /**
         * Events
         */
        this.OnHide = null
        this.OnShow = null

        this.OnDisable = () => {
            let i = setInterval( () => {
                let jq = $('#' + this.id)
                if(jq.length)
                {
                    clearInterval(i)
                    jq.block({message: null})
                }
            }, 100)
        }

        this.OnEnable = () => {
            let i = setInterval( () => {
                let jq = $('#' + this.id)
                if(jq.length)
                {
                    clearInterval(i)
                    jq.unblock()
                }
            }, 100)
        }
        
    }

    SetComponent(value) {
        this.component = value
        return this
    }

    SetId(value) {
        this.id = value
        return this
    }

    SetOnHide(value) {
        this.OnHide = value
        return this
    }

    SetOnDisable(value) {
        this.OnDisable = value
        return this
    }

    AddHtmlClass(value) {
        if(_.isString(value))
        {
            _.each(value.split(' '), item => this.htmlClass.Put(item, true))
        }
        else
        {
            if(_.isObject(value))
            {
                _.each(value, (v, k) => this.htmlClass.Put(k, v))
            }
        }
        return this
    }

    get html_class() {
        return this.htmlClass.All()
    }

    get html_style() {
        return this.htmlStyle.All()
    }

    Hide(vue = null) {
        if(this.enabled)
        {
            this.visible = false
            if(_.isFunction(this.OnHide))
            {
                this.OnHide(vue)
            }
        }
        return this
    }

    Show() {
        if(this.enabled)
        {
            this.visible = true
            if(_.isFunction(this.OnShow))
            {
                this.OnShow()
            }
        }
        return this
    }

    ToggleVisible() {
        return this.visible ? this.Hide() : this.Show()
    }

    Disable() {
        if(this.visible)
        {
            this.enabled = false
            if(_.isFunction(this.OnDisable))
            {
                this.OnDisable()
            }
        }
        return this
    }

    Enable() {
        if(this.visible)
        {
            this.enabled = true
            if(_.isFunction(this.OnEnable))
            {
                this.OnEnable()
            }
        }
        return this
    }

    ToggleEnabled() {
        return this.enabled ? this.Disable() : this.Enable()
    }

    Toggle(property) {
        if(property == 'visible')
        {
            this.ToggleVisible()
        }
        else
        {
            if(property == 'enabled')
            {
                this.ToggleEnabled()
            }
        }
        return this
    }
}

export default BaseComponent