import Card from "../Base/Card";
import Collection from "../../../Classes/Collection/Collection";
import AxiosCrud from "./../../../Classes/Axios/Crud/AxiosCrud"

class CrudCard extends Card {

    constructor(name) {
        super(name)

        /**
         * insert | update | delete
         */
        this.action = null

        /**
         * the initial record
         */
        this.old = null

        /**
         * the blank record
         */
        this.blank = null

        /**
         * the work record
         */
        this.record = null

        /**
         * controls collection
         */
        this.controls = new Collection()

        /**
         * Layout Button Declansator 
         */
        this.actionButtons = {
            insert: {
                icon: 'la la-plus',
                caption: 'Add',
                htmlClass: 'btn btn-primary btn-sm',
            },
            update: {
                icon: 'la la-save',
                caption: 'Save',
                htmlClass: 'btn btn-primary btn-sm',
            },
            delete: {
                icon: 'la la-trash',
                caption: 'Delete',
                htmlClass: 'btn btn-danger btn-sm',
            },
        }

        this.repeat_insert_action = true

        /**
         * crud axios object
         */
        this.axios = new AxiosCrud()

        /**
         * Daca avem fisier sau nu in record
         */
        this.has_file = false
        
        this.onSuccessResponse = (vue) => {}

        /**
         * Clasa HTML aplicata cardului
         */
        this.AddHtmlClass({
            'kt-portlet': true, 
            'kt-portlet--mobile': true,
            'kt-portlet--bordered': true,
        })

        /**
         * Cardul are footer
         */
        this.SetHasFooter(true)

        /**
         * Initial cardul nu este vizibil
         */
        this.Hide()
    }

    SetBlankRecord(record) {
        Vue.set(this, 'blank', {})
        Vue.set(this, 'old', {})
        Vue.set(this, 'record', {})
        _.each(record, (value, key) => {
            Vue.set(this.blank, key, value)
            Vue.set(this.old, key, value)
            Vue.set(this.record, key, value)
        })
        return this
    }

    AddControl(name, control) {
        this.controls.Put(name, control)
        return this
    }

    SetEndpoints(endpoints) {
        this.axios.SetEndpoints(endpoints)
        return this
    }

    SetOnSuccessResponse(value) {
        this.onSuccessResponse = value
        return this
    }

    SetRepeatedInsert(value) {
        this.repeat_insert_action = value
        return this
    }

    get cardControls() {
        return this.controls.All()
    }

    get cardActionsButton() {
        if(! this.action)
        {
            return null
        }
        if(! this.actionButtons.hasOwnProperty(this.action) )
        {
            return null
        }
        return this.actionButtons[this.action]
    }

    get cardCaption() {
        return this.caption(this.action, this.old)
    }

    get endpoint() {
        if(! this.action )
        {
            return null
        }
        return this.axios.endpoints[this.action]
    }

    ResetRecord() {
        _.each(this.record, (value, key) => Vue.set(this.record, key, this.blank[key]))
    }

    Show(action, old = null) {
        this.action = action
        if(old)
        {
            _.each(old, (value, key) => Vue.set(this.old, key, value))
        }
        else
        {
            _.each(this.blank, (value, key) => Vue.set(this.old, key, value))
        }
        _.each(this.old, (value, key) => {
            if(this.blank.hasOwnProperty(key))
            {
                Vue.set(this.record, key, value)
            }
        })
        if(_.isFunction(this.BeforeShow))
        {
            this.BeforeShow(action, old)
        }
        super.Show()
    }
    
}

export default CrudCard