import Control from "../../Control";

class Imagebox extends Control {

    constructor(name) {
        super(name)

        this.label = 'My Label (' + this.name + ')'

        this.allowed_extensions = null
        this.min_size = null
        this.max_size = null

        this.errors = null

    }

    SetLabel(value) {
        this.label = value
        return this
    }

    SetAllowedExtensions(value) {
        this.allowed_extensions = value
        return this
    }

    SetSizeLimits(min, max) {
        this.min_size = min
        this.max_size = max
        return this
    }

    ValidateFile(file) {
        if(file && (file != undefined))
        {
            Vue.set(this, 'errors', null)

            let size = file.size
            let parts = file.name.split('.'), extension = parts[parts.length-1].toLowerCase()
    
            let r = {
                vSize: {
                    valid: true,
                    message: null,
                    size,
                    min_size: this.min_size,
                    max_size: this.max_size,
                },
                vExtension: {
                    valid: true,
                    message: null,
                    extension,
                    allowed_extensions: this.allowed_extensions,
                },
    
            }
            
            /**
             * Validate size
             */
            if(this.min_size || this.max_size)
            {
                if(this.min_size && (size < this.min_size))
                {
                    r.vSize.valid = false
                    r.vSize.message = 'Mărimea fișierului (' + size + ') este mai mică decât dimensiunea minimă admisă (' + this.min_size + ').'
                }
                else
                {
                    if(this.max_size && (size > this.max_size))
                    {
                        r.vSize.valid = false
                        r.vSize.message = 'Mărimea fișierului (' + size + ') este mai mare decât dimensiunea maximă admisă (' + this.min_size + ').'
                    }
                }
            }
    
            let exists = _.find(this.allowed_extensions, item => item == extension)
            if( ! exists || (exists == undefined))
            {
                r.vExtension.valid = false
                r.vExtension.message = 'Extensia fișierul (' + extension + ') nu este permisă. Extensiile acceptate sunt: ' + this.allowed_extensions.join(', ')
            }
    
            if(! r.vSize.valid || ! r.vExtension.valid)
            {
                let errors = []
                if(! r.vSize.valid )
                {
                    errors.push(r.vSize.message)
                }
                if(! r.vExtension.valid )
                {
                    errors.push(r.vExtension.message)
                }
                Vue.set(this, 'errors', errors)
                return false
            }
            return true
        }
        return true
    }
}

export default Imagebox